package com.hungraim.ltc.system.controller;


import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.hungraim.ltc.constant.Gender;
import com.hungraim.ltc.pojo.entity.disable.ChFndOrgan;
import com.hungraim.ltc.pojo.entity.disable.ChSrvOrgan;
import com.hungraim.ltc.pojo.entity.system.SystemUser;
import com.hungraim.ltc.pojo.entity.system.SystemUserRole;
import com.hungraim.ltc.pojo.vo.system.ReqUserInfo;
import com.hungraim.ltc.system.dao.SystemUserRoleMapper;
import com.hungraim.ltc.system.service.ISystemUserRoleService;
import com.hungraim.ltc.system.service.ISystemUserService;
import com.hungraim.ltc.system.service.impl.SystemInterfaceServiceImpl;
import com.hungraim.ltc.system.service.impl.SystemUserRoleServiceImpl;
import com.hungraim.ltc.system.service.impl.SystemUserServiceImpl;
import com.hungraim.ltc.util.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author hubin
 */
@RestController
@RequestMapping("/api.system/user")
@Slf4j
public class UserController {





    private final ISystemUserService userInfoService;
    private final ISystemUserRoleService userRoleService;

    private final SystemInterfaceServiceImpl systemInterfaceService;

    @Resource
    private SystemUserRoleMapper systemUserRoleMapper;
    @Autowired
    public UserController(SystemUserServiceImpl userInfoService, SystemUserRoleServiceImpl userRoleService, SystemInterfaceServiceImpl systemInterfaceService) {
        this.userInfoService = userInfoService;
        this.userRoleService = userRoleService;
        this.systemInterfaceService = systemInterfaceService;
    }


    /**
     * 根据用户账号获取用户基本信息
     *
     * @param account 用户账号
     * @return 营销员信息
     */
    @GetMapping("/loadUserInfoByAccount/{account}")
    public Result<SystemUser> loadUserInfoByAccount(@PathVariable String account) {
        log.info("user:{} 请求登陆。。。", account);
        SystemUser systemUser = userInfoService.getOne(new LambdaQueryWrapper<SystemUser>().eq(SystemUser::getAccount, account));
        if (systemUser == null) {
            log.warn("user:{} 账号未找到！", account);
            return Result.failed(ResultCode.USER_NOT_EXIST);
        }
        return Result.success(systemUser);
    }

    /**
     * 根据用户id获取用户权限
     *
     * @param id 用户id
     * @return 权限列表
     */
    @GetMapping("/loadUserRolesByUserId/{id}")
    public Result loadUserRolesByUserId(@PathVariable Long id) {
        List<Long> roleIds = userRoleService.list(new LambdaQueryWrapper<SystemUserRole>()
            .eq(SystemUserRole::getUserId, id)
        ).stream().map(SystemUserRole::getRoleId).collect(Collectors.toList());
        log.info("user:{},拥有的权限:{}", id, roleIds);
        return Result.success(roleIds);
    }


    /**
     * 新增用户
     */
    @PostMapping("/addUser")
    public Result addUser(ReqUserInfo reqUserInfo) {
        if (StrUtil.isBlankIfStr(reqUserInfo.getAccount())) {
            return Result.failed("账号不能为空！");
        }
        if (StrUtil.isBlankIfStr(reqUserInfo.getPassword())) {
            return Result.failed("密码不能为空！");
        }
        if (StrUtil.isBlankIfStr(reqUserInfo.getName())) {
            return Result.failed("昵称不能为空！");
        }
        if (CheckStrength.checkPasswordStrength(reqUserInfo.getPassword()) < 4) {
            return Result.failed("密码太简单，请设置复杂密码！");
        }
        SystemUser systemUser = userInfoService.getOne(new LambdaQueryWrapper<SystemUser>()
            .eq(SystemUser::getAccount, reqUserInfo.getAccount()));
        if (systemUser != null) {
            return Result.failed("账号已存在！");
        }
        SystemUser user = new SystemUser();
        user.setAccount(reqUserInfo.getAccount());
        user.setPassword(new BCryptPasswordEncoder().encode(reqUserInfo.getPassword()));
        user.setGender(reqUserInfo.getGender());
        user.setRealName(reqUserInfo.getName());
        user.setUserStatus(reqUserInfo.getEnable());
        user.setCreatedTime(new Date());
        user.setOrganId(reqUserInfo.getOrganId());
        user.setSrvOrganId(reqUserInfo.getSrvOrganId());
        userInfoService.save(user);
        Long userId = user.getUserId();
        //初始化权限
        List<Long> roles = reqUserInfo.getRoleIds();
        roles.add(1L);
        List<SystemUserRole> roleList = roles.stream().distinct().map(roleId -> {
            SystemUserRole systemUserRole = new SystemUserRole();
            systemUserRole.setUserId(userId);
            systemUserRole.setRoleId(roleId);
            return systemUserRole;
        }).collect(Collectors.toList());
        userRoleService.saveBatch(roleList);
        systemInterfaceService.listResourceInterface();
        return Result.success("新增成功");
    }


    /**
     * 获得密码强度等级，包括简单、一般、复杂、强、非常强
     *
     * @param password
     * @return
     */
    @GetMapping("/getPasswordLevel/{password}")
    public Result getPasswordLevel(@PathVariable String password) {
        return Result.success(CheckStrength.getPasswordLevel(password));
    }

    /**
     * 判断账户是否合法
     *
     * @param account
     * @return
     */
    @GetMapping("/isLegitimate/{account}")
    public Result isLegitimate(@PathVariable String account) {
        if (StrUtil.isBlankIfStr(account)) {
            return Result.failed("账号不能为空！");
        }
        if (account.length() < 4) {
            return Result.failed("账号长度不能低于4位！");
        }
        if (!account.matches("^[a-z0-9A-Z_]+$")) {
            return Result.failed("账号只能由字母、数字、下划线组成！");
        }
        SystemUser systemUser = userInfoService.getOne(new LambdaQueryWrapper<SystemUser>()
            .eq(SystemUser::getAccount, account));
        if (systemUser != null) {
            return Result.failed("账号已存在！");
        }
        return Result.success("成功");
    }


    /**
     * 查询用户列表
     *
     * @param reqUserInfo
     * @return
     */
    @PostMapping("/queryUsers")
    public Result queryUsers(@RequestHeader("Authorization") String token,ReqUserInfo reqUserInfo) {
        long srvOrganId = TokenParsingUtil.TokenParsing(token, "srvOrganId");
        long organId = TokenParsingUtil.TokenParsing(token, "organId");

        QueryWrapper<SystemUser> systemUserQueryWrapper = new QueryWrapper<>();
        if (!StrUtil.isBlankIfStr(reqUserInfo.getName())) {
            systemUserQueryWrapper.and(wrapper -> wrapper.lambda().like(SystemUser::getRealName, reqUserInfo.getName()).or()
                .like(SystemUser::getAccount, reqUserInfo.getName()));
        }
        if (!StrUtil.isBlankIfStr(reqUserInfo.getGender())) {
            systemUserQueryWrapper.and(wrapper -> wrapper.lambda().eq(SystemUser::getGender, reqUserInfo.getGender()));
        }
        if (!StrUtil.isBlankIfStr(reqUserInfo.getEnable())) {
            systemUserQueryWrapper.and(wrapper -> wrapper.lambda().eq(SystemUser::getUserStatus, reqUserInfo.getEnable()));
        }
        if (srvOrganId!=0) {
            systemUserQueryWrapper.and(wrapper -> wrapper.lambda().eq(SystemUser::getSrvOrganId, srvOrganId));
        }
        if (organId!=0) {
            systemUserQueryWrapper.and(wrapper -> wrapper.lambda().eq(SystemUser::getOrganId, organId));
        }
        List<SystemUser> users = userInfoService.list(systemUserQueryWrapper);
        List<ChFndOrgan> chFndOrganList = userInfoService.fndAllOrgans();
        List<ChSrvOrgan> chSrvOrganList = userInfoService.srvOrganLists();

        List<ReqUserInfo> resUsers = users.stream().map(user -> {
            ReqUserInfo userInfo = new ReqUserInfo();
            userInfo.setId(user.getUserId());
            userInfo.setName(user.getRealName());
            userInfo.setAccount(user.getAccount());
            userInfo.setEnable(user.getUserStatus());
            userInfo.setAvatar(user.getAvatar());
            userInfo.setGender(Gender.MAN.getCode().equals(user.getGender()) ? Gender.MAN.getDesc() : Gender.WOMAN.getDesc());
            userInfo.setOrganId(user.getOrganId());
            userInfo.setSrvOrganId(user.getSrvOrganId());
            if(userInfo.getOrganId()!=null && userInfo.getOrganId()>0 && userInfo.getSrvOrganId()!=null && !userInfo.getSrvOrganId().equals("0")){
                List<ChFndOrgan> collect = chFndOrganList.stream().filter(chFndOrgan -> chFndOrgan.getOrganId().equals(user.getOrganId().toString())).collect(Collectors.toList());
                List<ChSrvOrgan> chSrvOrgans = chSrvOrganList.stream().filter(chSrvOrgan -> chSrvOrgan.getSrvOrganId().equals(user.getSrvOrganId().intValue())).collect(Collectors.toList());
                userInfo.setOrganName(collect.get(0).getOrganName());
                userInfo.setSrvOrganName(chSrvOrgans.get(0).getSrvOrganName());
            }
            return userInfo;
        }).collect(Collectors.toList());
        return Result.success(resUsers);
    }



    /**
     * 修改用户
     */ @PostMapping("/updateUser")
    public Result updateUser(ReqUserInfo reqUserInfo) {
        if (reqUserInfo.getId() == null || reqUserInfo.getId() == 0) {
            return Result.failed("用户id无效！");
        }
        SystemUser systemUser = userInfoService.getOne(new LambdaQueryWrapper<SystemUser>().eq(SystemUser::getUserId, reqUserInfo.getId()));
        if (systemUser == null) {
            return Result.failed("用户id无效！");
        }
        boolean flag = false;
        if (!reqUserInfo.getName().equals(systemUser.getRealName())) {
            systemUser.setRealName(reqUserInfo.getName());
            flag = true;
        }
        if (!reqUserInfo.getAccount().equals(systemUser.getAccount())) {
            systemUser.setAccount(reqUserInfo.getAccount());
            flag = true;
        }
        if (!reqUserInfo.getGender().equals(systemUser.getGender())) {
            systemUser.setGender(reqUserInfo.getGender());
            flag = true;
        }
        if (!reqUserInfo.getId().equals(systemUser.getUserId())) {
            systemUser.setUserId(reqUserInfo.getId());
            flag = true;
        }
        if (flag) {
            systemUser.setUpdatedTime(new Date());
            userInfoService.updateById(systemUser);
        }
        //查询该角色下所有的资源数据
        List<SystemUserRole> roleResources = userRoleService.list(new LambdaQueryWrapper<SystemUserRole>().eq(SystemUserRole::getUserId, reqUserInfo.getId()));

        if(reqUserInfo.getRoleIds()==null){
            reqUserInfo.setRoleIds(new ArrayList<>());
        }
        List<SystemUserRole> newRoleResources = reqUserInfo.getRoleIds().stream()
            .filter(id -> {
                for (SystemUserRole r : roleResources) {
                    if (r.getRoleId().equals(id)) {
                        return false;
                    }
                }
                return true;
            }).map(resourceId -> {
                SystemUserRole systemRoleResource = new SystemUserRole();
                systemRoleResource.setRoleId(resourceId);
                systemRoleResource.setUserId(reqUserInfo.getId());
                return systemRoleResource;
            }).collect(Collectors.toList());
        userRoleService.saveBatch(newRoleResources);
        List<Long> deleteRoleResources = roleResources.stream()
            .map(SystemUserRole::getRoleId).filter(id -> !reqUserInfo.getRoleIds().contains(id))
            .collect(Collectors.toList());
        if(!deleteRoleResources.isEmpty()){
            systemUserRoleMapper.removeByIds(deleteRoleResources,reqUserInfo.getId());
        }
        systemInterfaceService.listResourceInterface();
       // systemUserRoleMapper.removeByIds(deleteRoleResources,reqUserInfo.getId());
       // userRoleService.removeByIds(deleteRoleResources);
        return Result.success("更新成功");
    }
    /**
     * 根据id 删除用户
     */
    @PostMapping("/deleateByUserId")
    public Result deleateByUserid(Long id){
        this.userInfoService.removeById(id);
        return Result.success("更新成功");
    }

    @PostMapping("/updatePassword")
    public Result updatePassword(String oldPassword,String newPassword,@RequestHeader("Authorization") String token){
        log.info("oldPassword:"+oldPassword+",newPassword:"+newPassword);
        if(StringUtils.isBlank(newPassword) || StringUtils.isBlank(oldPassword)){
            Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }

        //校验旧密码是否匹配
        Integer userId = TokenParsingUtil.TokenParsing(token, "userId");
        if(userId == null){
            return Result.failed("未获取到用户信息！");
        }
        SystemUser userinfo = userInfoService.getById(userId);
        if(userinfo == null){
            return Result.failed("未获取到用户信息！");
        }
        BCryptPasswordEncoder bCryptPasswordEncoder = new BCryptPasswordEncoder();

        String userinfoPassword = userinfo.getPassword();

        if(!bCryptPasswordEncoder.matches(oldPassword, userinfoPassword)){
            return Result.failed("旧密码不匹配！");
        }


        if (CheckStrength.checkPasswordStrength(newPassword) < 4) {
            return Result.failed("密码太简单，请设置复杂密码！");
        }


        SystemUser user = new SystemUser();
        user.setPassword(new BCryptPasswordEncoder().encode(newPassword));
        user.setUserId(userId.longValue());
        userInfoService.updateById(user);

        return Result.success();
    }


}
